/*
 * Generates a *.m3u playlist file from a directory by sorting the entrys after file modification time.
 * 
 * www.sunshine2k.de || www.bastian-molkenthin.de
 * webmaster@sunshine2k.de
 * 
 * March, 2009
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace PlaylistGen
{

    public partial class Form1 : Form
    {

        // this is our list with all filename, sorted by filetime
        private List<KeyValuePair<String, DateTime>> pairList;
        // only files with following extensions are recognized as music files
        public string[] Extensions = new string[] { ".MP3", ".OGG", ".WAV", ".WMA" };
        // this is the user's chosen directory
        private String basePath;

        public Form1()
        {
            InitializeComponent();
            // create our main list holding all files
            pairList = new List<KeyValuePair<string, DateTime>>();
        }

        
        private void ChooseDirButton_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            fbd.Description = "Please choose a folder";

            if (fbd.ShowDialog() == DialogResult.OK)
            {
                pairList.Clear();
                basePath = fbd.SelectedPath;
                GetFilesAndSort(fbd.SelectedPath, RecursiveDirCheckBox.Checked, RelativePathCheckBox.Checked);
                MessageBox.Show("Parsing Done!", ":-)", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void SavePlaylistButton_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.AddExtension = true;
            sfd.DefaultExt = ".m3u";
            sfd.Filter = "M3U Files|*.m3u|All Files|*.*";
            sfd.OverwritePrompt = true;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                if (SaveAsPlaylist(sfd.FileName))
                    MessageBox.Show("File saved!", "", MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show("File could not be saved!", "Error occured!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        public bool isValidExtension(string extension)
        {
            extension = extension.ToUpper();
            foreach (string ext in Extensions)
            {
                if (extension.Equals(ext))
                    return true;
            }
            return false;
        }

        public void GetFilesAndSort(string directory, bool recursive, bool relpaths)
        {
            // get all files in directory
            DirectoryInfo dirInfo = new DirectoryInfo(directory);
            FileInfo[] files = dirInfo.GetFiles();

            // insert all files with modified time in our list
            foreach (FileInfo f in files)
            {
                // only add music files to list
                if (isValidExtension(f.Extension))
                {
                    if (!relpaths)      // absolute paths
                        pairList.Add(new KeyValuePair<String, DateTime>(f.FullName, f.LastWriteTime));
                    else
                    {
                        // relative paths
                        pairList.Add(new KeyValuePair<String, DateTime>(
                            f.FullName.Replace(basePath+"\\", ""), f.LastWriteTime));
                    }
                }
                                    

            }

            // parse subdirectories if necessary
            if (recursive)
            {
                foreach (DirectoryInfo d in dirInfo.GetDirectories())
                    GetFilesAndSort(d.FullName, recursive, relpaths);
            }

            // sort our list by date
            pairList.Sort(
               delegate(KeyValuePair<String, DateTime> a, KeyValuePair<String, DateTime> b)
               {
                   return (a.Value.CompareTo(b.Value));
               }
            );

        }

        public bool SaveAsPlaylist(string filename)
        {
            // create m3u file, even if it already exists
            bool result = false;
            try
            {
                TextWriter writer = new StreamWriter(filename, false, Encoding.Default);
                writer.WriteLine("#EXTM3U");
                foreach (KeyValuePair<String, DateTime> pair in pairList)
                    writer.WriteLine(pair.Key);
                writer.Close();
                result = true;
            }
            catch (Exception)
            {
                result = false;
            }
            return result;
        }

        private void InfoButton_Click(object sender, EventArgs e)
        {
            MessageBox.Show(
                "This little tool parses a given directory for music files" + System.Environment.NewLine +
                "and saves them to a M3U playlist file sorted by " + System.Environment.NewLine +
                "file modification time in ascending order (oldest first)." + System.Environment.NewLine +
                System.Environment.NewLine +
                "It was quickly hacked for personal needs as winamp playlist" + System.Environment.NewLine +
                "window cannot sort the files by file date." + System.Environment.NewLine +
                "Maybe it's also useful for you" + System.Environment.NewLine +
                System.Environment.NewLine +
                "Sunshine, March 2k9" + System.Environment.NewLine +
                "www.sunshine2k.de || www.bastian-molkenthin.de", "Info",
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

    }
}